@@ -1,7 +1,7 @@
 
 package Confluence::Client::XMLRPC;
 {
-  $Confluence::Client::XMLRPC::VERSION = '2.3';
+  $Confluence::Client::XMLRPC::VERSION = '2.4';
 }
 use strict;
 use warnings;
@@ -177,6 +177,41 @@ sub updatePage {
 	return $result;
 }
 
+#sub updatePageAfterFetch
+## above method does no work with confluence 3.5.17 if the page already exits.
+## this version fetches the page, and updates the old page if it exists,
+## otherwise writes the new page.  RT 2/1/14.
+#{
+#      my Confluence::Client::XMLRPC $self = shift;
+#      my ($newPage)                       = @_;
+#
+#      my $saveRaise                       = setRaiseError(0);
+#
+#      my $result = "";
+#      my $oldPage = $self->getPage( $newPage->{space}, $newPage->{title} );
+#
+#      if (ref($oldPage) eq 'HASH') {
+#              if (defined($oldPage->{space}) and defined($oldPage->{title})
+#                              and $newPage->{space} eq $oldPage->{space}
+#                              and $newPage->{title} eq $oldPage->{title}
+#              )
+#              {
+#                      $newPage->{id}      = $oldPage->{id};
+#                      $newPage->{version} = $oldPage->{version};
+#                      $result             = $self->storePage($newPage);
+#              }
+#      } else {
+#              $result = $self->storePage($newPage);
+#      }
+#
+#      setRaiseError($saveRaise);
+#      if ($LastError) {
+#              croak $LastError if $RaiseError;
+#              warn $LastError  if $PrintError;
+#      }
+#      return $result;
+#}
+
 sub _rpc {
 	my Confluence::Client::XMLRPC $self = shift;
 	my $method = shift;
@@ -251,7 +286,7 @@ Confluence::Client::XMLRPC - Client for the Atlassian Confluence wiki, based on
 
 =head1 VERSION
 
-version 2.3
+version 2.4
 
 =head1 SYNOPSIS
 
@@ -263,41 +298,51 @@ version 2.3
 
   my $object = Confluence::Client::XMLRPC->new( <URL>, <user>, <pass>, 2 );
 
-=head1 CAVEAT
+=head1 METHODS
 
-B<ATTENTION>, please: This module was written by Asgeir Nilsen in 2004 and later on
-improved by Giles Lewis, Martin Ellis, and Torben K. Jensen.
+All method calls are simply mapped (via C<AUTOLOAD>) to RPC method calls.
+Please refer to 
+L<the official list of available methods|https://developer.atlassian.com/display/CONFDEV/Remote+Confluence+Methods>
+for further information.
 
-I - Heiko Jansen - only took the available source code and created a CPAN distribution
-for it, because at least to me a Perl module almost does not exist if it's not on 
-available via CPAN.
+The module tries to automatically map between the data types mentioned in 
+the Atlassian docs and the appropriate Perl data types:
 
-This package B<should> work with any remote API function.
+=over 4
 
-The original authors tested it with C<addUserToGroup>, C<getActiveUsers>, C<getPage>, C<getPages>, C<getServerInfo>,
-C<getUser>, and C<storePage>. I (Heiko Jansen) have used it successfully to create and update pages, but I did
-B<not> test most other API functions and am thus B<unable to give any guarantee that it will work as expected>!
+=item C<Vectors> : array references
 
-The original module was simply named "Confluence" but since Atlassian is currently
-working on a new REST-based API and since there already is L<Jira::Client> and
-L<Jira::Client::REST> on CPAN I renamed it to C<Confluence::Client::XMLRPC>.
+=item C<Structs> and C<data objects> : hash references
+
+=item C<Boolean> : strings "true" and "false"
+
+=back
+
+For everything else, simple scalars are used and mapped to RPC::XML::string, 
+so explicit type conversions should not be required.
+
+B<Note>: the "token" paramater mentioned in the Confluence docs is the session
+id and is automatically added by this module, so do not pass in a parameter 
+for it when invoking a method.
 
 =head1 ERROR HANDLING
 
-This package has two global flags which control error handings.
+This package has two global flags which control error handling.
 
   Confluence::Client::XMLRPC::setRaiseError(1);  # Enable die
   Confluence::Client::XMLRPC::setPrintError(1);  # Enable warn
   Confluence::Client::XMLRPC::setRaiseError(0);  # Disable die
   Confluence::Client::XMLRPC::setPrintError(0);  # Disable warn
 
-The C<setRaiseError> and C<setPrintError> functions both return the previous setting of the flag so that it may be restored if necessary.
+The C<setRaiseError> and C<setPrintError> functions both return the previous 
+setting of the flag so that it may be restored if necessary.
 
 RaiseError is initially set to 1 to preserve the original package behavior.
 
 PrintError is initially set to 0.
 
-If RaiseError is set to 0 then C<Confluence::Client::XMLRPC::lastError()> can be used to determine if an error occurred.
+If RaiseError is set to 0 then C<Confluence::Client::XMLRPC::lastError()> 
+can be used to determine if an error occurred.
 
   Confluence::Client::XMLRPC::setRaiseError(0);
   my $page = $wiki->getPage($space, $title);
@@ -305,7 +350,13 @@ If RaiseError is set to 0 then C<Confluence::Client::XMLRPC::lastError()> can be
       say $e;
   }
 
-=head1 API VERSION
+=head1 Debugging
+
+You can get more info about the communication between your client and the
+API by setting the environment variable C<CONFLDEBUG> to a true value.
+If you do so, the module will log the messages exchanged to STDERR.
+
+=head1 API VERSIONS
 
 Analogous to the global error handling flags there is a flag to 
 set the API version to use:
@@ -319,24 +370,28 @@ version namespaces ("confluence1", "confluence2").
 
 The version 2 of the API was introduced with Confluence 4.0 and 
 B<Atlassian recommends to use the newer version>.
-
 However, due to backwards compatibility reasons the default value for the 
-API version in this module still is B<1>. Note: you can use B<most
-but not all> of the version 1 API calls on newer Confluence installations!
+API version in this module still is B<1>. 
 
-For a detailed and authoritative description of the differences between 
-versions 1 and 2 of the API please refer to
-L<https://developer.atlassian.com/display/CONFDEV/Confluence+XML-RPC+and+SOAP+APIs#ConfluenceXML-RPCandSOAPAPIs-v2apiRemoteAPIversion1andversion2>!
+Note: you can use B<most but not all> of the version 1 API calls on newer 
+Confluence installations! The Confluence docs contain a detailed and 
+authoritative description of 
+L<the differences between versions 1 and 2 of the API|https://developer.atlassian.com/display/CONFDEV/Confluence+XML-RPC+and+SOAP+APIs#ConfluenceXML-RPCandSOAPAPIs-v2apiRemoteAPIversion1andversion2>!
 
-=head1 USAGE
+The new version 2 API implements the same methods as the version 1 API, 
+however all content is stored and retrieved using the storage format. 
+This means that you cannot, for example, create a page using wiki markup 
+with the version 2 API, you must instead define the page using the new
+XHTML based storage format.
+You will be able to create pages, blogs and comments in wiki markup 
+using the version 1 API even on Confluence 4.0 and later. However you 
+will no longer be able to retrieve pages, blogs and comments using 
+the version 1 API.
 
-=head2 Data types
-
-Perl simple data types are mapped to string.
-Hash references are mapped to struct.
-Array references are mapped to array.
-
-This package now automatically converts all scalars to RPC::XML::string, so explicit type conversions should not be required.
+To aid in the migration phase Confluence 4.0 and up provide a method
+C<convertWikiToStorageFormat()> where you can pass in a string with 
+wiki markup and will recieve the same data converted to the new storage 
+format (which you can then use to create or update a page).
 
 =head1 API extension
 
@@ -344,12 +399,14 @@ This package now automatically converts all scalars to RPC::XML::string, so expl
 
 =item updatePage
 
-This package has a function called C<updatePage> which is not part of the original remote API.
-If the page id is not specified then the function will call C<storePage> to do an insert. 
-If an "already exists" error is encountered then the function will call C<getPage> to retrieve
-the page id and version, and then repeat the C<storePage> attempt. This function is intended 
-to be used in situations where the intent is to upload pages, overwriting existing content if 
-it exists. See example below.
+This package has a function called C<updatePage> which is not part of the 
+original remote API. If the page id is not specified then the function will 
+call C<storePage> to do an insert. 
+If an "already exists" error is encountered then the function will call 
+C<getPage> to retrieve the page id and version, and then repeat the 
+C<storePage> attempt. This function is intended to be used in situations 
+where the intent is to upload pages, overwriting existing content if it 
+exists. See example below.
 
 =back
 
@@ -359,32 +416,71 @@ it exists. See example below.
 
 =item C<upload_files.pl> - Upload files
 
-The sample script uploads the contents of a directory to the wiki. Each file in the directory 
-is uploaded as a separate page. The page title is the file name with extension removed. This 
-script requires five arguments: API url, user name, password, space key and a directory name.
+The sample script uploads the contents of a directory to the wiki. Each file 
+in the directory is uploaded as a separate page. The page title is the file 
+name with extension removed. This script requires five arguments: API url, 
+user name, password, space key and a directory name.
 
 =item C<upload_users.pl> - Upload Users
 
-This script reads and loads a list of users from a file (or stdin). If errors are encountered then 
-the script will print an error message, but continue processing.
-This script requires three arguments: API url, name and password of an admin user.
+This script reads and loads a list of users from a file (or stdin). If errors 
+are encountered then the script will print an error message, but continue 
+processing.
+This script requires three arguments: API url, name and password of an admin 
+user.
 
 =item C<det_group_mbrship.pl> - Determine Group Membership
 
 The script prints the group membership of all users.
-This script requires three arguments: API url, name and password of an admin user.
+This script requires three arguments: API url, name and password of an admin 
+user.
 
 =back
 
-Please refer to the C<examples> directory of the distribution for the scripts themselves.
+Please refer to the C<examples> directory of the distribution for the scripts 
+themselves.
 
-=head1 SEE ALSO
+=head1 NOTES
 
-The package uses the L<RPC::XML> module to do the heavy lifting. Read the perldoc for this package to learn more.
+The package uses the L<RPC::XML> module to do the heavy lifting. Read the 
+perldoc for this package to learn more.
+
+L<RPC::XML> uses LWP for handling C<http>/C<https> messaging. If you are 
+experiencing problems when connecting to a C<https> based API endpoint,
+please make sure that the necessary modules - like, e.g. 
+L<LWP::Protocol::https> - are installed.
 
 For further information on the Confluence API itself please refer to the 
-L<official documentation|https://developer.atlassian.com/display/CONFDEV/Confluence+XML-RPC+and+SOAP+APIs> as provided
-by Atlassian.
+L<official documentation|https://developer.atlassian.com/display/CONFDEV/Confluence+XML-RPC+and+SOAP+APIs>
+as provided by Atlassian.
+
+Please note that starting with Confluence 5.5 the XML-RPC API will be 
+deprecated, meaning that Atlassian will not add new features or fix bugs 
+related to the XML-RPC API for Confluence 5.5 or later.
+This does B<not> mean, that this module will not work with newer Confluence
+versions: as of now, there is no information if or when Atlassian will remove
+the XML-RPC API and rely solely on the new REST API.
+
+=head1 CAVEAT
+
+B<ATTENTION>, please: This module was written by Asgeir Nilsen in 2004 and 
+later on improved by Giles Lewis, Martin Ellis, and Torben K. Jensen.
+
+I - Heiko Jansen - only took the available source code and created a CPAN 
+distribution for it, because at least to me a Perl module almost does not 
+exist if it's not on available via CPAN.
+
+This package B<should> work with any remote API function.
+
+The original authors tested it with C<addUserToGroup>, C<getActiveUsers>, 
+C<getPage>, C<getPages>, C<getServerInfo>, C<getUser>, and C<storePage>. 
+I (Heiko Jansen) have used it successfully to create and update pages, but 
+I did B<not> test most other API functions and am thus B<unable to give any 
+guarantee that it will work as expected>!
+
+The original module was simply named "Confluence" but since Atlassian is 
+currently working on a new REST-based API I renamed it to 
+C<Confluence::Client::XMLRPC>.
 
 =head1 AUTHORS
 
